@@ -16,12 +16,17 @@ import time
 from joblib import Parallel, delayed
 import copy
 from Bio import Entrez, Medline
+from pyvis.network import Network
+import re
 
-from transformers import AutoTokenizer, AutoModel
+import nltk
+nltk.download('punkt')
+
+from transformers import AutoTokenizer, AutoModel, AutoModelForSequenceClassification
 from transformers import pipeline
 
 rel_tokenizer = AutoTokenizer.from_pretrained("JacopoBandoni/BioBertRelationGenesDiseases")
-rel_model = AutoModel.from_pretrained("JacopoBandoni/BioBertRelationGenesDiseases")
+rel_model = AutoModelForSequenceClassification.from_pretrained("JacopoBandoni/BioBertRelationGenesDiseases")
 
 rel_model.eval()
 
@@ -30,7 +35,7 @@ rel_model = rel_model.to(device)
 
 rel_pipe = pipeline(task='text-classification', model=rel_model, tokenizer=rel_tokenizer, device=0 if device=='cuda' else -1)
 
-def download_articles_biopython(title: str, start_year: int, end_year: int, max_results: int = 100, author: str = '') -> List[Article]:
+def download_articles_biopython(title: str, start_year: int, end_year: int, max_results: int = 100, author: str = '', type_research :str = 'relevance') -> List[Article]:
     """
     Download articles from PubMed using BioPython library.
     Args:
@@ -51,14 +56,19 @@ def download_articles_biopython(title: str, start_year: int, end_year: int, max_
             return pickle.load(f)
 
     Entrez.email = 'pubmadbiosearch@gmail.com'
-    query = '(' + title + '[Title]) AND ' + '(' + author + '[Author])' + ' AND ' + '(("' + str(start_year) + '"[Date - Create] : "' + \
-                           str(end_year) + '"[Date - Create]))'
+
+    if author != "":
+        query = '(' + title + ') AND (' + author + '[Author])'
+    else:
+        query = title 
 
     handle = Entrez.esearch(db='pubmed', 
-                            sort='first+author', 
+                            sort=type_research, 
                             retmax=max_results,
-                            retmode='xml',
-                            usehistory='y', 
+                            retmode='xml', 
+                            datetype='pdat',
+                            mindate=str(start_year),
+                            maxdate=str(end_year),
                             term=query)
     results = Entrez.read(handle)               
     id_list = results['IdList']
@@ -68,13 +78,15 @@ def download_articles_biopython(title: str, start_year: int, end_year: int, max_
     results = Medline.parse(handle)
     results = list(results)
     articles = []
+    print(results[1])
     for article in results:
         abstract = article.get("AB", "?")
-        if abstract != "?":
+        title = article.get("TI", "?")
+        if abstract != "?" and title != "?":
             if abstract.find("This article has been withdrawn at the request of the author(s) and/or editor") == -1 and \
                abstract.find("An amendment to this paper has been published and can be accessed via a link at the top of the paper") == -1 and \
-               abstract.find("This corrects the article") == -1 and abstract.find("A Correction to this paper has been published") == -1:
-                articles.append(Article(title=article['TI'], abstract=abstract, 
+               abstract.find("This corrects the article") == -1 and abstract.find("A Correction to this paaper has been published") == -1:
+                articles.append(Article(title=title, abstract=abstract, 
                                         pmid=article['PMID'], full_text='', publication_data=datetime.strptime(article['DP'][:4], '%Y')))
 
     print("Found {} articles".format(len(articles)))
@@ -155,15 +167,22 @@ def extract_entities(article: Article, source: str = 'abstract') -> List[Entity]
     # Query BERN2 server for entities.
     extracted = False
     bern_result = None
-    while not extracted:
+
+    i = 0
+
+    while not extracted and i < 3:
         try:
             bern_result = query_plain(text)
             extracted = True
         except Exception as e:
             print("Error querying BERN2 server")
-            print("Sleep for 10 seconds and try again to not get banned by BERN2")
-            time.sleep(10)
+            print("Sleep for 3 seconds and try again to not get banned by BERN2")
+            time.sleep(3)
+            i += 1
             continue
+    
+    if i == 3:
+        return []
 
     # Parse the result.
     annotations = bern_result['annotations']
@@ -392,4 +411,58 @@ def find_entity(entity: Entity, span_sentences: List[List[int]]) -> int:
     for i in range(len(span_sentences)):
         if span_sentences[i][0] <= entity.span_begin and entity.span_end <= span_sentences[i][1]:
             return i
-    return -1
\ No newline at end of file
+    return -1
+
+def add_title_node(G, n, d):
+  result_html = f"""
+  <h3>{d['mention']}</h3>
+  """
+  #articles: 
+  #  <ul>
+  #"""
+  #pmids = re.split("[,\\n]", d['pmid'])
+  #for pmid in pmids:
+  #  result_html += f"<li>{pmid}</li>"
+  #result_html += "</ul>"
+  return result_html
+  
+def add_title_edge(G, edge):
+  #find intersection between pmids of the two nodes
+  node1 = dict(G.nodes(data=True))[edge[0]]
+  node2 = dict(G.nodes(data=True))[edge[1]]
+  n1_pmids = re.split(",",node1['pmid'])
+  n2_pmids = re.split(",",node2['pmid'])
+  intersection = list(set(n1_pmids) & set(n2_pmids))
+
+  #create html 
+  result_html = f"""<p>common articles between <strong>{node1['mention']}</strong> and <strong>{node2['mention']}</strong></p>
+  
+  <ul>"""
+  for pmid in intersection:
+    result_html += f"<li>{pmid}</li>"
+  result_html += "</ul>"
+  return result_html
+  
+
+def html_graph(G, name="nodes"):
+  net = Network(notebook=True)
+
+  #add nodes
+  for n, d in G.nodes(data=True):
+    color = '#ccc'
+    if d['type'] == 'gene':
+      color = '#0DA3E4'
+    elif d['type'] == 'disease':
+      color = '#bf4d2d'
+    net.add_node(n, d['mention'], color=color, title= add_title_node(G, n, d))
+
+  #add edges
+  for edge in G.edges(data='True'):
+    add_title_edge(G,edge)
+    weight = G.get_edge_data(edge[0], edge[1])['weight']
+    net.add_edge(edge[0], edge[1], value=weight, color='#F0EB5A', title=add_title_edge(G,edge))
+
+  net.show_buttons(filter_=['physics'])
+  net.show(name + ".html")
+
+  return net;
